"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.labelize = exports.estimateLabelMaxSize = void 0;
const simple_pure_utils_1 = require("simple-pure-utils");
const tokenizer_1 = require("./tokenizer");
function isHexToken(x) {
    const r = x;
    return r.size != undefined && r.hex != undefined;
}
function isLabelDef(x) {
    return x.type == "labelBegin" || x.type == "labelEnd";
}
function estimateLabelMaxSize(tokens) {
    const contractSizeWithoutLabels = (0, simple_pure_utils_1.sum)(tokens.map(x => isHexToken(x) ? x.size : 0));
    const labelCount = tokens.filter(x => x.type == "labelOp").length;
    const maxTheoricalLabelSize = 15;
    // we need to safisfy 
    // labelSize >= log256(contractSizeWithoutLabels + (1 byte + labelSize) * labelCount)
    for (let labelSize = 1; labelSize < maxTheoricalLabelSize; labelSize++) {
        const contractSize = contractSizeWithoutLabels + (1 + labelSize) * labelCount;
        const contractSizeBytes = Math.log(contractSize) / Math.log(256);
        if (labelSize >= contractSizeBytes) {
            return labelSize;
        }
    }
    return maxTheoricalLabelSize;
}
exports.estimateLabelMaxSize = estimateLabelMaxSize;
function labelize(tokens) {
    const stack = [];
    const labels = {};
    // define labels:
    let index = 0;
    const labelSize = estimateLabelMaxSize(tokens);
    for (const token of tokens) {
        if (token.type == "labelBegin") {
            stack.push({
                index,
                name: token.name
            });
            index = 0;
        }
        if (token.type == "labelEnd") {
            const label = stack.pop();
            if (label == null) {
                throw new Error("Stack is empty at index " + index);
            }
            index = label.index + index;
            const size = index - label.index;
            labels[label.name] = {
                index: label.index,
                size: size
            };
        }
        if (isHexToken(token)) {
            index += token.size;
        }
        if (token.type == "labelOp") {
            index += 1; //push
            index += labelSize;
        }
    }
    return (0, simple_pure_utils_1.flatten)(tokens
        .filter(x => !isLabelDef(x))
        .map(x => {
        if (isHexToken(x))
            return [x];
        if (isLabelDef(x))
            throw new Error("Unreachable");
        const label = labels[x.label];
        if (label == null)
            throw new Error(`Label '${x.label}' not found`);
        if (x.op == "dataOffset") {
            return (0, tokenizer_1.pushConstant)((0, tokenizer_1.toHex)(label.index, labelSize));
        }
        if (x.op == "dataSize") {
            return (0, tokenizer_1.pushConstant)((0, tokenizer_1.toHex)(label.size, labelSize));
        }
        (0, simple_pure_utils_1.assertUnreachable)(x.op);
    }));
}
exports.labelize = labelize;
