"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const test_helper_1 = require("./test-helper");
const tokenizer_1 = require("./tokenizer");
test("simple ops", () => {
    const actual = (0, tokenizer_1.tokenize)(`
    push1
    push1
    mstore
    `);
    const expected = [
        (0, test_helper_1.op)("push1"),
        (0, test_helper_1.op)("push1"),
        (0, test_helper_1.op)("mstore"),
    ];
    expect(actual).toEqual(expected);
});
test("op consts", () => {
    const actual = (0, tokenizer_1.tokenize)(`
    0x80
    0x40
    mstore
    `);
    const expected = [
        ...(0, tokenizer_1.pushConstant)("0x80"),
        ...(0, tokenizer_1.pushConstant)("0x40"),
        (0, test_helper_1.op)("mstore"),
    ];
    expect(actual).toEqual(expected);
});
test("op call", () => {
    const actual = (0, tokenizer_1.tokenize)(`
    mstore(0x40, 0x80)
    `);
    const expected = [
        ...(0, tokenizer_1.pushConstant)("0x80"),
        ...(0, tokenizer_1.pushConstant)("0x40"),
        (0, test_helper_1.op)("mstore"),
    ];
    expect(actual).toEqual(expected);
});
test("spaces", () => {
    const code = "tag_1:\npop";
    expect((0, tokenizer_1.reduceOpSpaces)(code)).toEqual(code);
});
test("label", () => {
    const actual = (0, tokenizer_1.tokenize)(`
tag_1:
pop
    `);
    const expected = [
        {
            type: "labelBegin",
            name: "tag_1"
        },
        {
            type: "labelEnd"
        },
        (0, test_helper_1.op)("jumpdest"),
        (0, test_helper_1.op)("pop")
    ];
    expect(actual).toEqual(expected);
});
test("auxdata", () => {
    const actual = (0, tokenizer_1.tokenize)(`
auxdata: 0x1234
    `);
    const expected = [
        (0, test_helper_1.op)("invalid"),
        {
            type: "const",
            size: 2,
            hex: "0x1234"
        }
    ];
    expect(actual).toEqual(expected);
});
test("code block", () => {
    const actual = (0, tokenizer_1.tokenize)(`
sub_0: assembly {
    dup1
}
    `);
    const expected = [
        {
            type: "labelBegin",
            name: "sub_0"
        },
        (0, test_helper_1.op)("dup1"),
        {
            type: "labelEnd"
        }
    ];
    expect(actual).toEqual(expected);
});
test("labelRef", () => {
    const actual = (0, tokenizer_1.tokenize)(`
dup1
label_0
dup2
label_0:
dup3
dup4
    `);
    const expected = [
        (0, test_helper_1.op)("dup1"),
        {
            type: "labelOp",
            label: "label_0",
            op: "dataOffset"
        },
        (0, test_helper_1.op)("dup2"),
        {
            type: "labelBegin",
            name: "label_0"
        },
        {
            type: "labelEnd"
        },
        (0, test_helper_1.op)("jumpdest"),
        (0, test_helper_1.op)("dup3"),
        (0, test_helper_1.op)("dup4"),
    ];
    expect(actual).toEqual(expected);
});
